package de.szut.lf8_starter.integration.project;

import de.szut.lf8_starter.project.ProjectRepository;
import org.json.JSONObject;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDate;
import java.util.Arrays;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
public class CreateProjectActionTest {
    @Autowired
    private MockMvc mockMvc;
    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void createProjectTest() throws Exception {
        String content = """
                {
                    "name": "name",
                    "leading_employee": 1,
                    "employees": [2, 3],
                    "contractor": 4,
                    "contractor_name": "Peter File",
                    "comment": "goal of project",
                    "start_date": "01.01.2000",
                    "planned_end_date": "01.01.2001"
                  }
                """;

        final var contentAsString = this.mockMvc.perform(
                post("/projects").content(content).contentType(MediaType.APPLICATION_JSON)
        )
                .andExpect(status().isCreated())
                .andExpect(jsonPath("name", is("name")))
                .andExpect(jsonPath("leading_employee", is(1)))
                .andExpect(jsonPath("employees", is(Arrays.asList(2, 3))))
                .andExpect(jsonPath("contractor", is(4)))
                .andExpect(jsonPath("contractor_name", is("Peter File")))
                .andExpect(jsonPath("comment", is("goal of project")))
                .andExpect(jsonPath("start_date", is("01.01.2000")))
                .andExpect(jsonPath("planned_end_date", is("01.01.2001")))
                .andReturn()
                .getResponse()
                .getContentAsString();

            final var id = Long.parseLong(new JSONObject(contentAsString).get("id").toString());

            final var project = this.projectRepository.findById(id);
            assertThat(project.get().getName()).isEqualTo("name");
            assertThat(project.get().getLeadingEmployee()).isEqualTo(1);
            assertThat(project.get().getContractor()).isEqualTo(4);
            assertThat(project.get().getContractorName()).isEqualTo("Peter File");
            assertThat(project.get().getComment()).isEqualTo("goal of project");
            assertThat(project.get().getStartDate()).isEqualTo(LocalDate.of(2000, 1, 1));
            assertThat(project.get().getPlannedEndDate()).isEqualTo(LocalDate.of(2001, 1, 1));
    }
}
